/*
ST7032.h

Modified by Kenichi KAKIZAKI
2017/9/24
2015/12/27
2015/12/20
2015/11/03
2015/05/08
*/

#ifndef _ST7032_H
#define _ST7032_H

//#include <inttypes.h>
#include <Print.h>

#define	ST7032_ADDR	0x3e

// POWER CONTROL
#define	PWR5V 0
#define	PWR3V3 1

#define	ST7032_DATA_CONTRAST5V	12
#define	ST7032_DATA_CONTRAST3V3	32

// commands
#define ST7032_CMD_CLEARDISPLAY 0x01
#define ST7032_CMD_RETURNHOME 0x02
#define ST7032_CMD_ENTRYMODESET 0x04
#define ST7032_CMD_DISPLAYCONTROL 0x08
#define ST7032_CMD_CURSORSHIFT 0x10
#define ST7032_CMD_FUNCTIONSET 0x30
#define ST7032_CMD_SETCGRAMADDR 0x40
#define ST7032_CMD_SETDDRAMADDR 0x80


#define ST7032_CMD_OSCFREQ 0x10
#define ST7032_CMD_POWERCONTRAST 0x50
#define ST7032_CMD_FOLLOWER 0x60
#define ST7032_CMD_CONTRAST 0x70

// flags for display entry mode
#define ST7032_CMD_ENTRYRIGHT 0x00
#define ST7032_CMD_ENTRYLEFT 0x02
#define ST7032_CMD_ENTRYSHIFTINCREMENT 0x01
#define ST7032_CMD_ENTRYSHIFTDECREMENT 0x00

// flags for display on/off control
#define ST7032_CMD_DISPLAYON 0x04
#define ST7032_CMD_DISPLAYOFF 0x00
#define ST7032_CMD_CURSORON 0x02
#define ST7032_CMD_CURSOROFF 0x00
#define ST7032_CMD_BLINKON 0x01
#define ST7032_CMD_BLINKOFF 0x00

// flags for display/cursor shift
#define ST7032_CMD_DISPLAYMOVE 0x08
#define ST7032_CMD_CURSORMOVE 0x00
#define ST7032_CMD_MOVERIGHT 0x04
#define ST7032_CMD_MOVELEFT 0x00

// flags for function set
#define ST7032_CMD_2LINE 0x08
#define ST7032_CMD_1LINE 0x00
#define ST7032_CMD_5x10DOTS 0x04
#define ST7032_CMD_5x8DOTS 0x00
#define ST7032_CMD_EIS 0x01

// flags for booster set
#define ST7032_CMD_BOOSTER 0x04


class ST7032 : public Print {
public:
#if defined(ARDUINO_ARCH_ESP32) || defined(ARDUINO_ARCH_ESP8266)
  ST7032(uint8_t sda, uint8_t scl);
  ST7032();

  void init(uint8_t scl, uint8_t sda);
#elif defined(ARDUINO_ARCH_SAM) || defined(ARDUINO_ARCH_SAMD)
  ST7032();

  void init();
#else
  ST7032();
  ST7032(uint8_t pwr);

  void init(uint8_t pwr);
#endif
  void begin(uint8_t cols, uint8_t rows, uint8_t charsize = ST7032_CMD_5x8DOTS);

  void clear();
  void home();

  void noDisplay();
  void display();
  void noBlink();
  void blink();
  void noCursor();
  void cursor();
  void scrollDisplayLeft();
  void scrollDisplayRight();
  void leftToRight();
  void rightToLeft();
  void autoscroll();
  void noAutoscroll();
  void setContrast(uint8_t);
  uint8_t getContrast();

  void setRowOffsets(int row1, int row2, int row3, int row4);
  void createChar(uint8_t, uint8_t[]);
  void setCursor(uint8_t, uint8_t); 
  virtual size_t write(uint8_t);
  void command(uint8_t);
  
  using Print::write;

private:
  void send(uint8_t, uint8_t);

  uint8_t _addr ;
#if defined(ARDUINO_ARCH_ESP8266)
  uint8_t _scl ;
  uint8_t _sda ;
#endif
  uint8_t _power ;

  uint8_t _displayfunction;
  uint8_t _displaycontrol;
  uint8_t _displaymode;

  uint8_t _initialized;

  uint8_t _numlines;
  uint8_t _row_offsets[4];

  uint8_t _contrast;
};

#endif
